@@ -52,7 +52,7 @@ def deep_merge(target : dict, data : dict) -> None:
             target[key2] = value2
 
 
-def select(data : dict, path : str, default_value : typing.Any = None) -> dict:
+def select(data : typing.Union[dict, Object, object], path : str, default_value : typing.Any = None) -> typing.Any:
     paths = path.split(".")
     for path in paths:
         if isinstance(data, dict) and path in data:
@@ -65,103 +65,76 @@ def select(data : dict, path : str, default_value : typing.Any = None) -> dict:
             return default_value
     return data
 
-def touch(data, path, default_value):
+def touch(data : typing.Union[dict, Object, object], path : str, default_value : typing.Any) -> typing.Any:
+    """Make sure data has the path.
+    If data has the path, return the orignal value.
+    If data NOT has the path, create a new path, and set to default_value, returns the default value.
+    """
     result = select(data, path, _NULL)
     if result == _NULL:
         update(data, path, default_value)
-        return select(data, path, None)
+        return default_value
     else:
         return result
 
-def update(data : dict, path : str, value : typing.Any) -> dict:
+def attrgetorset(data : typing.Union[dict, Object, object], key : str, default_value : typing.Any) -> typing.Any:
+    """Get or set attr to data directly, and the key is a one level key.
+    If data contains the key, get the original value.
+    If data NOT contains the key, set the key to the default value.
+    """
+    if isinstance(data, dict):
+        if not key in data:
+            data[key] = default_value
+        return data[key]
+    elif isinstance(data, list):
+        key = int(key)
+        if key >= len(data):
+            for _ in range(key + 1 - len(data)):
+                data.append(None)
+            data[key] = default_value
+        return data[key]
+    else:
+        if not hasattr(data, key):
+            setattr(data, key, default_value)
+        return getattr(data, key)
+
+def attrset(data : typing.Union[dict, Object, object], key : str, value : typing.Any) -> None:
+    """Set attr to data directory, and the key is a one level key.
+    If data contains the key, overrdie the original value with the new value.
+    If data NOT contains the key, add a new key to the new value.
+    """
+    if isinstance(data, dict):
+        data[key] = value
+    elif isinstance(data, list):
+        key = int(key)
+        if key >= len(data):
+            for _ in range(key + 1 - len(data)):
+                data.append(None)
+        data[key] = value
+    else:
+        setattr(data, key, value)
+
+def update(data : typing.Union[dict, Object, object], path : str, value : typing.Any) -> typing.Union[dict, Object, object]:
+    """Set attr to data, and the key is a dot-seperated-path.
+    If data contains the key, override the original value with the new value.
+    If data NOT contains the key, add a new key to the new value.
+    """
+    old_data = data
+    is_object = isinstance(data, Object)
     paths = path.split(".")
     for index in range(0, len(paths) - 1):
         path = paths[index]
         path_next = paths[index + 1]
-
-        if isinstance(data, dict) and path in data:
-            data = data[path]
-        elif isinstance(data, list) and int(path) < len(data):
-            data = data[int(path)]
+        if path_next.isdigit():
+            next_empty_value = []
         else:
-            if isinstance(data, list) and int(path) >= len(data):
-                for _ in range(int(path) + 1 - len(data)):
-                    data.append(None)
-            if path_next.isdigit():
-                next_value = []
+            if is_object:
+                next_empty_value = Object()
             else:
-                next_value = {}
-            if path.isdigit():
-                data[int(path)] = next_value
-                data = data[int(path)]
-            else:
-                data[path] = next_value
-                data = data[path]
+                next_empty_value = {}
+        data = attrgetorset(data, path, next_empty_value)
     path = paths[-1]
-    if path.isdigit():
-        for _ in range(int(path) + 1 - len(data)):
-            data.append(None)
-        data[int(path)] = value
-    else:
-        data[path] = value
-    return
-    
-
-
-def update(data : typing.Union[dict, Object], path : str, value : typing.Any) -> typing.Union[dict, Object]:
-
-    old_data = data
-
-    if isinstance(data, Object):
-        is_object = True
-    else:
-        is_object = False
-
-    def is_forloop_last(index, data):
-        if index == len(data) - 1:
-            return True
-        else:
-            return False
-            
-    def insert_empty(index, paths, data, path):
-        empty = {}
-        if is_object:
-            empty = Object()
-        if not is_forloop_last(index, paths):
-            next_path = paths[index+1]
-            if next_path.isdigit():
-                empty = []
-        data[path] = empty
-        return data[path]
-
-    paths = path.split(".")
-    last_path = paths[-1]
-    paths = paths[:-1]
-    len_paths = len(paths)
-    for index, path in enumerate(paths):
-        if isinstance(data, dict) and path in data:
-            data = data[path]
-        elif isinstance(data, (list, tuple)) and path.isdigit():
-            if int(path) < len(data):
-                data = data[int(path)]
-            else:
-                for _ in range(int(path) + 1 - len(data)):
-                    data.append(None)
-                data = insert_empty(index, paths, data, int(path))
-        elif hasattr(data, path):
-            data = getattr(data, path)
-        else:
-            data = insert_empty(index, paths, data, path)
-    if isinstance(data, (list, tuple)) and last_path.isdigit():
-        if int(last_path) < len(data):
-            data[int(last_path)] = value
-        else:
-            for _ in range(int(last_path) + 1 - len(data)):
-                data.append(None)
-            data[int(last_path)] = value
-    else:
-        data[last_path] = value
-
+    attrset(data, path, value)
     return old_data
 
 def ignore_none_item(data: dict) -> dict:
